//
//  GamesViewController.h
//  LottoTouch
//
//  Created by Bertils mini on 2008-03-23.
//  Copyright 2008-2009 Sork Sofware. All rights reserved.
//

#import "GamesViewController.h"
#import "PickerViewController.h"
#import "CustomCell.h"
#import "Defines.h"


@interface GamesViewController (Private)
- (NSDictionary *)createNewGame;
- (void)saveGamesData;
- (void)updateGamesList;
- (void)presetsChanged:(NSNotification *)notification;
- (void)tableView:(UITableView *)tableView didSelectEditedRowWithIndexPath:(NSIndexPath *)indexPath;
@end


@implementation GamesViewController

@synthesize gamesList, gamesDictionary;
@synthesize selectedTitle, newGame, lotto1Size, sample1Size, lotto2Size, sample2Size;


- (id)init
{
	if (self = [super init]) {
		// This will appear as the title in the navigation bar
		self.title = NSLocalizedString(@"GamesViewTitle", @"");
	}
	
	return self;
}

- (void)dealloc
{
	[gamesTableView release];
	[gamesList release];
	[pickerViewController release];
	[notificationCenter removeObserver:self];
	[super dealloc];
}

- (void)loadView
{	
	// Create a plain table view with one section that fills the content view (or put it directly on the main screen)
	gamesTableView = [[UITableView alloc] initWithFrame:[[UIScreen mainScreen] applicationFrame] style:UITableViewStylePlain];
	gamesTableView.delegate = self;
	gamesTableView.dataSource = self;	
	gamesTableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
	gamesTableView.allowsSelectionDuringEditing = YES;

	self.view = gamesTableView;

	// Insert default Edit bar button that invokes setEditing:animated:
	self.navigationItem.rightBarButtonItem = self.editButtonItem;
}

// Update the key list and sort it
- (void)updateGamesList {
	// List all the games in the dictionary supplied by the Lotto View Controller
	self.gamesList = [NSMutableArray arrayWithArray:[gamesDictionary allKeys]];
	// Sort the game titles alphabetically
	[self.gamesList sortUsingSelector:@selector(caseInsensitiveCompare:)];
}

// Called by presetsChanged: and by tableView:commitEditingStyle:forRowAtIndexPath:
- (void)saveGamesData
{
	// The Lotto View Controller will save the change(s) to disk
	NSDictionary *presetsChanged = [NSDictionary dictionaryWithObject:@"Game" forKey:@"Change"];
	
	[notificationCenter postNotificationName:SSLottoTouchPresetsChangedNotification
									  object:self userInfo:presetsChanged];
}

// Private method used when user touches a table row - didSelectRowAtIndexPath:
// - or the green plus editing control - commitEditingStyle: in editing mode
- (void)tableView:(UITableView *)tableView didSelectEditedRowWithIndexPath:(NSIndexPath *)indexPath
{
	UITableViewCell *selectedCell = [gamesTableView cellForRowAtIndexPath:indexPath];

	// Modify an existing game?
	if (!self.newGame) {
		// Yes, get the touched game
		NSDictionary *game = [gamesDictionary objectForKey:[self.gamesList objectAtIndex:indexPath.row]];
		
		// Get the parameters of the touched game
		self.lotto1Size = [[game objectForKey:LOTTO1_SIZE] intValue];
		self.sample1Size = [[game objectForKey:SAMPLE1_SIZE] intValue];
		self.lotto2Size = [[game objectForKey:LOTTO2_SIZE] intValue];
		self.sample2Size = [[game objectForKey:SAMPLE2_SIZE] intValue];
		
		// Update the seleced row to the touched one
		self.selectedTitle = selectedCell.textLabel.text;
	}
	
	// Create the picker view lazily
	if (pickerViewController == nil) {
		pickerViewController = [[PickerViewController alloc] init];
		
		// Listen to updates from the picker view
		[notificationCenter addObserver:self selector:@selector(presetsChanged:)
								   name:SSLottoTouchPresetsChangedNotification object:nil];
	}
	
	// Transfer parameters to the picker view
	pickerViewController.newGame = self.newGame;
	pickerViewController.lotto1Size = self.lotto1Size;
	pickerViewController.sample1Size = self.sample1Size;
	pickerViewController.lotto2Size = self.lotto2Size;
	pickerViewController.sample2Size = self.sample2Size;
	
	// A New Game requires special handling
	if (self.newGame) {
		pickerViewController.gameTitle = selectedCell.textLabel.text;
		// Ensure minimum values for a new game
		if (self.lotto1Size == 0) {
			pickerViewController.lotto1Size = SAMPLE_SIZE_MIN;
			pickerViewController.sample1Size = 1;
		}
	} else {
		pickerViewController.gameTitle = self.selectedTitle;
	}
	
	// Finally, drill down to the picker view
	[[self navigationController] pushViewController:pickerViewController animated:YES];
}

#pragma mark -
#pragma mark Notifications

- (void)presetsChanged:(NSNotification *)notification
{
	// Get the change type from the notification userInfo
	NSString *prefsType = [[notification userInfo] objectForKey:@"Change"];
	
	// Save changes in the Picker view – and the Title view
	if ([prefsType isEqualToString:@"Save"]) {		
		// Get the new game settings from picker view
		self.lotto1Size = pickerViewController.lotto1Size;
		self.sample1Size = pickerViewController.sample1Size;
		self.lotto2Size = pickerViewController.lotto2Size;
		self.sample2Size = pickerViewController.sample2Size;
		
		// Replace existing game or add new one with the changed or new params
		[gamesDictionary setObject:[self createNewGame] forKey:pickerViewController.gameTitle];
		// The new game becomes the selected one
		self.selectedTitle = pickerViewController.gameTitle;
				
		// Save new and old to file
		[self saveGamesData];
	}
}

// Create new game dict to insert in the games dictionary
- (NSDictionary *)createNewGame
{
	NSDictionary *dict = [NSDictionary dictionaryWithObjectsAndKeys:
						  [NSString stringWithFormat:@"%u", self.lotto1Size], @"lotto1Size",
						  [NSString stringWithFormat:@"%u", self.sample1Size], @"sample1Size",
						  [NSString stringWithFormat:@"%u", self.lotto2Size], @"lotto2Size",
						  [NSString stringWithFormat:@"%u", self.sample2Size], @"sample2Size", nil];
	return dict;
}

#pragma mark -
#pragma mark UIViewController methods

// Invoked when the user touches the Edit/Done button
- (void)setEditing:(BOOL)editing animated:(BOOL)animated {
	[self.navigationItem setHidesBackButton:editing animated:YES];
	
    // Updates the appearance of the Edit/Done button as necessary.
    [super setEditing:editing animated:animated];
    [gamesTableView setEditing:editing animated:YES];
	
	// Reload view to show/hide the insert (+) row
	[gamesTableView reloadData];
}

- (void)viewWillAppear:(BOOL)animated
{
	[self updateGamesList];
		
	[gamesTableView reloadData];
}

- (void)viewDidAppear:(BOOL)animated
{
	self.newGame = NO;
	
	// iPhone table views do not display a persistent selected state
	NSIndexPath *tableSelection = [gamesTableView indexPathForSelectedRow];
	[gamesTableView deselectRowAtIndexPath:tableSelection animated:YES];
}

#pragma mark -
#pragma mark UITableView delegate methods

// Tells the delegate that the specified row is now selected
- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	UITableViewCell *selectedCell = [tableView cellForRowAtIndexPath:indexPath];

	if (!self.editing) {
		// Not editing, reset old selection
		[[tableView cellForRowAtIndexPath:oldIndexPath] setAccessoryType:UITableViewCellAccessoryNone];
		// Set checkmark on new selection
		selectedCell.accessoryType = UITableViewCellAccessoryCheckmark;
		
		oldIndexPath = indexPath;

		// Set the new title
		self.selectedTitle = selectedCell.textLabel.text;
				
		// Let the Lotto View controller know about a change
		NSDictionary *presetsChanged = [NSDictionary dictionaryWithObject:@"Game" forKey:@"Change"];
		
		[notificationCenter postNotificationName:SSLottoTouchPresetsChangedNotification 
										  object:self userInfo:presetsChanged];
		
		// Return to the lotto view
		[self.navigationController popViewControllerAnimated:YES];
	} else {
		// Editing, did the user choose a new game?
		if (indexPath.row == ([self.gamesList count])) {
			// Yes, set new flag
			self.newGame = YES;
		}
		
		// Then go to the Picker view (private method) for new and existing games
		[self tableView:tableView didSelectEditedRowWithIndexPath:indexPath];
	}
}

// Asks the delegate for the editing style of a particular row
- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath
{
	// This row will disable swipe delete
	UITableViewCellEditingStyle editingStyle = UITableViewCellEditingStyleNone;
	
	// Enable delete & insert during editing
	if (self.editing) {
		// Is this the extra insert row?
		if (indexPath.row == ([self.gamesList count])) {
			editingStyle = UITableViewCellEditingStyleInsert;
		} else {
			editingStyle = UITableViewCellEditingStyleDelete;
		}
	}
	return editingStyle;
}

// The table view has left editing mode
- (void)tableView:(UITableView *)tableView didEndEditingRowAtIndexPath:(NSIndexPath *)indexPath
{
	// Show the Back button again
	[self.navigationItem setHidesBackButton:NO animated:YES];

	// Enable the right bar button again
	self.navigationItem.rightBarButtonItem.enabled = YES;
}

#pragma mark -
#pragma mark UITableView DataSource methods

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	if (self.editing) {
		return [self.gamesList count] + 1;
	}
	return [self.gamesList count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	// Use a custom cell that overrides hidesAccessoryWhenEditing:
	CustomCell *cell = (CustomCell *)[tableView dequeueReusableCellWithIdentifier:CELL_IDENTIFIER];

	if (cell == nil) {
		cell = [[[CustomCell alloc] initWithFrame:CGRectZero reuseIdentifier:CELL_IDENTIFIER] autorelease];
		
		// Set initial cell accessory types
		cell.editingAccessoryType =  UITableViewCellAccessoryDisclosureIndicator;
		
		// The edited list will have one more item than the games list array
		if (indexPath.row < [self.gamesList count]) {
			if ([self.selectedTitle isEqualToString:[self.gamesList objectAtIndex:indexPath.row]]) {
				cell.accessoryType = UITableViewCellAccessoryCheckmark;
				oldIndexPath = indexPath;
			} else {
				cell.accessoryType = UITableViewCellAccessoryNone;
			}
		}
	}
	
	// Is this the cell that adds a new game?
	if (indexPath.row >= [self.gamesList count]) {
		// Yes, it will have a special label
		cell.textLabel.text = NSLocalizedString(@"NewGameListing", @"Add a new game in the games list");
		cell.selectionStyle = UITableViewCellSelectionStyleNone;
	} else {
		// No, set label to a games name
		cell.textLabel.text = [self.gamesList objectAtIndex:indexPath.row];
	}
	return cell;
}

// Ask the data source to commit the insertion or deletion of a specified game
// Invoked when the user taps the insertion (green plus) control or Delete button (after tapping the deletion (red minus) control)
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
	/* DELETE */
	// If row is deleted, remove the entry from the dictionary
    if (editingStyle == UITableViewCellEditingStyleDelete) {
		// First get the key
		NSString *gameToDelete = [self.gamesList objectAtIndex:indexPath.row];
		// Change the selection, if required
		if ([self.selectedTitle isEqualToString:gameToDelete]) {
			/* don't delete the last game!! */
			if ([self.gamesList count] == 1) {
				// NSLog(@"Can't delete last game!");
				return;
			}					
			// Reset selected to first free game
			int i, gamesInList = [self.gamesList count];
			for ( i = 0; i < gamesInList; i++ ) {
				if (gameToDelete != [self.gamesList objectAtIndex:i]) {
					self.selectedTitle = [self.gamesList objectAtIndex:i];
					break;
				}
			}
			// Inform the Lotto View controller about the change
			NSDictionary *presetsChanged = [NSDictionary dictionaryWithObject:@"Game" forKey:@"Change"];
			[notificationCenter postNotificationName:SSLottoTouchPresetsChangedNotification
											  object:self userInfo:presetsChanged];					
		}
		// Then delete the unwanted dictionary entry
		[gamesDictionary removeObjectForKey:gameToDelete];
		
		// Save the updated games dictionary
		[self saveGamesData];
		
		// Update list and view
		[self updateGamesList];
		[gamesTableView reloadData];
		
	/* INSERT */
	} else if (editingStyle == UITableViewCellEditingStyleInsert) {
		self.newGame = YES;
		// Private method
		[self tableView:tableView didSelectEditedRowWithIndexPath:indexPath];
	}
}

@end
