//
//  LottoMatrixView.m
//  LottoTouch
//
//  Created by Bertils mini on 2008-07-15.
//  Copyright Sork Software 2008-2009. All rights reserved.
//

#import "LottoMatrixView.h"
#import "LottoData.h"
#import "Defines.h"

NSString *SSLottoTouchMatrixChangedNotification = @"Matrix Clusions changed";


@interface LottoMatrixView(Private)
- (NSInteger)buttonIndexFromTouches:(NSSet *)touches;
- (UIImage *)makeStretchedButtonImage:(UIImage *)imageProto ofSize:(CGFloat)buttonSize;
@end


@implementation LottoMatrixView

@synthesize data;
@synthesize lottoSize, sampleSize;
@synthesize buttonSide, marginWidth, fontHeight;
@synthesize lottoFont;
@synthesize touchedImage, untouchedImage, lottedImage, includedImage, excludedImage;
@synthesize buttonsDictionary, cellMatrix;


- (id)initWithFrame:(CGRect)frame {
	if (self = [super initWithFrame:frame]) {
		// The buttons matrix presets from plist contains button sizes and font heights
		NSString *defaultBPPath = [[NSBundle mainBundle] pathForResource:@"ButtonsPresets" ofType:@"plist"];
		self.buttonsDictionary = [NSDictionary dictionaryWithContentsOfFile:defaultBPPath];
	}
	return self;
}

- (void)dealloc {
	[data release];
	[lottoFont release];
	[buttonsDictionary release];
	[cellMatrix release];
	[touchedImage release];
	[untouchedImage release];
	[lottedImage release];
	[includedImage release];
	[excludedImage release];
	[super dealloc];
}

#pragma mark -
#pragma mark Utility methods

// Called by LottoViewController's updateLottoViews
- (void)updateBoard
{
	// Calculate the size of a square button matrix - ie the number of buttons on each matrix side
	squares = (int)ceil(sqrt(lottoSize));
	
	NSString *squareKey = [NSString stringWithFormat:@"%u", squares];
	NSDictionary *buttonsData = [buttonsDictionary objectForKey:squareKey];
	self.buttonSide = [[buttonsData objectForKey:BUTTON_SIDE] intValue];
	self.fontHeight = [[buttonsData objectForKey:MATRIX_FONT_SIZE] intValue];
	self.marginWidth = (kScreenWidth - squares * buttonSide) / 2;

	self.lottoFont = [UIFont boldSystemFontOfSize:fontHeight];

	// Create and cache the five button images
	[self setTouchedImage:[self makeStretchedButtonImage:[UIImage imageNamed:@"touchedButtonProto.png"] ofSize:buttonSide]];
	[self setUntouchedImage:[self makeStretchedButtonImage:[UIImage imageNamed:@"untouchedButtonProto.png"] ofSize:buttonSide]];
	[self setLottedImage:[self makeStretchedButtonImage:[UIImage imageNamed:@"lottedButtonProto.png"] ofSize:buttonSide]];
	[self setIncludedImage:[self makeStretchedButtonImage:[UIImage imageNamed:@"includedButtonProto.png"] ofSize:buttonSide]];
	[self setExcludedImage:[self makeStretchedButtonImage:[UIImage imageNamed:@"excludedButtonProto.png"] ofSize:buttonSide]];
	
	// Create an array to hold the state of the button matrix
	[self setCellMatrix:[[NSMutableArray alloc] initWithCapacity:lottoSize]];
	// Setup the array for normal state buttons
	NSInteger i;
	NSNumber *state;
	for ( i = 0; i < lottoSize; i++ ) {
		state = [NSNumber numberWithInt:kButtonStateNormal]; // creates an autoreleased object
		[cellMatrix addObject:state];
	}
}

// Create a buttom image here. Thanks to FieryRobot! for this code snippet!
- (UIImage *)makeStretchedButtonImage:(UIImage *)imageProto ofSize:(CGFloat)stretchedSize
{
	UIImage* proto = [imageProto stretchableImageWithLeftCapWidth:8.0 topCapHeight:8.0];
	
	UIGraphicsBeginImageContext(CGSizeMake(stretchedSize, stretchedSize));	
	[proto drawInRect:CGRectMake(0.0, 0.0, stretchedSize, stretchedSize)];	
	UIImage *image = UIGraphicsGetImageFromCurrentImageContext();	
	UIGraphicsEndImageContext();
	
	return image;
}

#pragma mark -
#pragma mark Handling touches on the button matrix

// Start tracking a touch here
- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	// Get index for new touch
	NSInteger index = [self buttonIndexFromTouches:touches];
	
	// Touch inside button matrix?
	if (index >= 0) {
		// Yes, save the old button state before updating to the new state
		oldState = [[cellMatrix objectAtIndex:index] intValue];
		[cellMatrix replaceObjectAtIndex:index withObject:[NSNumber numberWithInt:kButtonStateTouched]];
		// Save the new button index too
		oldIndex = index;
		[self setNeedsDisplay];
	}
	// Touch was outside matrix
	else {
		oldIndex = -1;
	}
}

// Sent if the touch moves while still down
- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
	// Get index for new position
	NSInteger newIndex = [self buttonIndexFromTouches:touches];

	// Do nothing if touch is still over same button
	if (newIndex == oldIndex) return;
	
	// Is new position outside the button matrix?
	if (newIndex < 0) {
		// Yes, just return if it was outside before too
		if (oldIndex < 0) return;
		else {
			// No, it has just moved outside a button
			// Restore button state to that before touch
			[cellMatrix replaceObjectAtIndex:oldIndex withObject:[NSNumber numberWithInt:oldState]];
			// The touch is now outside matrix
			oldIndex = -1;
		}
	}
	else {
		// New position is inside the button matrix
		// Was the previous one inside too?
		if (oldIndex >= 0)
			// Yes, restore state of old button
			[cellMatrix replaceObjectAtIndex:oldIndex withObject:[NSNumber numberWithInt:oldState]];
		// Save previous state of the touched button
		oldState = [[cellMatrix objectAtIndex:newIndex] intValue];
		// Set new status of the touched button
		[cellMatrix replaceObjectAtIndex:newIndex withObject:[NSNumber numberWithInt:kButtonStateTouched]];
		oldIndex = newIndex;
	}
	[self setNeedsDisplay];
}

// Make something of the finished touch
- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	// Index of final position
	NSInteger index = oldIndex;

	// Touch still inside button matrix?
	if (index >= 0) {
		// Yes, change button state as needed
		switch (oldState) {
			// This is a normal number that will be included, increase the included count
			case kButtonStateNormal:
				if (includes == sampleSize) {
					// Return to normal state until an inclusion is removed
					[cellMatrix replaceObjectAtIndex:index withObject:[NSNumber numberWithInt:kButtonStateNormal]];
					break;
				}			
				[cellMatrix replaceObjectAtIndex:index withObject:[NSNumber numberWithInt:kButtonStateIncluded]];
				includes++;
				break;
			// This number was included, reduce the included count, will become excluded
			case kButtonStateIncluded:
				if (excludes == sampleSize) {
					// Return to previous state until an exlusion is removed
					[cellMatrix replaceObjectAtIndex:index withObject:[NSNumber numberWithInt:kButtonStateIncluded]];
					break;
				}
				[cellMatrix replaceObjectAtIndex:index withObject:[NSNumber numberWithInt:kButtonStateExcluded]];
				includes--;
				excludes++;
				break;
			// This number was excluded, restore to normal
			case kButtonStateExcluded:
				[cellMatrix replaceObjectAtIndex:index withObject:[NSNumber numberWithInt:kButtonStateNormal]];
				excludes--;
				break;
			default:
				break;
		}
		// Has the button been touched?
		if (includes || excludes) {
			// Yes, enable the Clear button
			NSDictionary *matrixChanged = [NSDictionary dictionaryWithObject:@"Enable" forKey:@"Touch"];
			[notificationCenter postNotificationName:SSLottoTouchMatrixChangedNotification
											  object:self userInfo:matrixChanged];
		} else {
			// No, disable the Clear button
			NSDictionary *matrixChanged = [NSDictionary dictionaryWithObject:@"Disable" forKey:@"Touch"];
			[notificationCenter postNotificationName:SSLottoTouchMatrixChangedNotification
											  object:self userInfo:matrixChanged];
		}
		[[self data] randomize];
		[self setNeedsDisplay];
	}
}

// The touch was cancelled by the operating system (by a phone call, most likely)
- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event
{
	// Restore button state to that before the cancelled touch
	[cellMatrix replaceObjectAtIndex:oldIndex withObject:[NSNumber numberWithInt:oldState]];
}

// Used by the LottoViewController's gameAction: to control the Clear button
// The method setupGamesFromFile also needs the 'cludes counts from the stored matrix
- (BOOL)hasBeenTouched
{
	BOOL touched = NO;
	includes = 0;
	excludes = 0;
	
	// A nice example of fast enumeration
	for (NSNumber *num in cellMatrix) {
		int buttonState = [num intValue];
		// Touched buttons are not normal
		if (buttonState != kButtonStateNormal) {
			touched = YES;
			// A touched button is either included or excluded
			if (buttonState == kButtonStateIncluded) includes++;
			else excludes++;
		}
	}
	return touched;
}

// Get the button number from a touch. Invalid touches return -1
- (NSInteger)buttonIndexFromTouches:(NSSet *)touches
{
	NSInteger index;
	UITouch *touch = [touches anyObject];
	CGPoint touchPoint = [touch locationInView:self];
	
	// Exclude touches outside the button matrix
	// This is the button matrix frame
	CGRect matrixFrame = CGRectMake(marginWidth, 
									marginWidth, 
									(self.frame.size.width - marginWidth * 2), 
									(self.frame.size.height - marginWidth * 2));
	
	// Is the touched point inside the button matrix?
	if (CGRectContainsPoint(matrixFrame, touchPoint)) {
		// Yes, calculate it's position
		NSUInteger column = (NSUInteger)(touchPoint.x - marginWidth) / buttonSide;
		NSUInteger row = (NSUInteger)(touchPoint.y - marginWidth) / buttonSide;
		index = row * squares + column;
		
		// Is the button index outside the lotto limit?
		if (index >= lottoSize) index = -1;
		// Return a valid index or an invalid one if outside
		return index;
	}
	// Touch was outside button matrix
	return -1;
}

// Reset the button matrix. Called by clearAction: in Lotto View controller
- (void)resetBoard
{
	includes = 0;
	excludes = 0;
	
	NSInteger i;
	for ( i = 0; i < lottoSize; i++ ) {
		[cellMatrix replaceObjectAtIndex:i withObject:[NSNumber numberWithInt:kButtonStateNormal]];
	}
}

#pragma mark -
#pragma mark Drawing the button matrix

// Draw the button matrix with proper highlighting
- (void)drawRect:(CGRect)rect {	
	float xPos, yPos;
    NSInteger i, n;
	NSInteger sample = sampleSize - includes;
    NSNumber *num;
    NSString *lott;
    CGSize lottSize;

	// Draw all the button images - one way ore another
    for ( i = 0; i < lottoSize; i++ ) {
		num = [[[self data] numbers] objectAtIndex:i];
        lott = [num stringValue];
        n = [num intValue] - 1;
        xPos = (n % squares * buttonSide) + marginWidth;
        yPos = (n / squares) * buttonSide + marginWidth;
		CGPoint buttonPoint = CGPointMake(xPos, yPos);
		// Default dark gray text
		[[UIColor colorWithHue:0.0 saturation:0.0 brightness:0.43 alpha:1.0] set];
		int currentState = [[cellMatrix objectAtIndex:n] intValue];
		switch (currentState) {
			case kButtonStateNormal:
				if (sample > 0) {
					[[self lottedImage] drawAtPoint:buttonPoint];
					[[UIColor whiteColor] set];
					sample--;
				} else {
					[[self untouchedImage] drawAtPoint:buttonPoint];
				}
				break;
			case kButtonStateTouched:
				[[self touchedImage] drawAtPoint:buttonPoint];
				break;
			case kButtonStateIncluded:
				[[self includedImage] drawAtPoint:buttonPoint];
				[[UIColor whiteColor] set];
				break;
			case kButtonStateExcluded:
				[[self excludedImage] drawAtPoint:buttonPoint];
				// Dark red text (Hue is 8°)
				[[UIColor colorWithHue:0.022 saturation:0.86 brightness:0.77 alpha:1.0] set];
				break;
			default:
				break;
		}		
        lottSize = [lott sizeWithFont:lottoFont];
		CGPoint textPoint = CGPointMake(xPos + (buttonSide - lottSize.width) / 2.0, yPos + (buttonSide - lottSize.height) / 2.0);
        [lott drawAtPoint:textPoint withFont:lottoFont];
    }
}


@end
