//
//  LottoTouchAppDelegate.m
//  LottoTouch
//
//  Created by Bertils mini on 2008-03-08.
//  Copyright Sork Software 2008-2009. All rights reserved.
//

#import "LottoTouchAppDelegate.h"
#import "LottoViewController.h"
#import "Defines.h"
#import "lottoData.h"
#import "LottoMatrixView.h"


@interface LottoTouchAppDelegate (Private)
- (void)createEditableCopyOfPresetsIfNeeded;
- (void)createUserLocationPresetsIfNeeded;
@end


@implementation LottoTouchAppDelegate

@synthesize window;


- (void)applicationDidFinishLaunching:(UIApplication *)application
{
	// The application ships with default presets in its bundle. If anything in the application
    // bundle is altered, the code sign will fail. We want the presets to be editable by users, 
    // so we need to create a copy of it in the application's Documents directory.     
    [self createEditableCopyOfPresetsIfNeeded];
	// The first time the application is used, an editable storage for the current user location
	// in the games section is created. This will later be modified as needed.
	[self createUserLocationPresetsIfNeeded];
		
	// Create the window
	self.window = [[[UIWindow alloc] initWithFrame:[[UIScreen mainScreen] bounds]] autorelease];
	
	// Set up the lotto view controller
    lottoController = [[LottoViewController alloc] init];
	
	// Create a navigation controller using the new controller
	navigationController = [[UINavigationController alloc] initWithRootViewController:lottoController];
	navigationController.navigationBar.barStyle = UIBarStyleDefault;
	// Make the nav bar green to match the lotto button bar
	navigationController.navigationBar.tintColor = [UIColor colorWithHue:0.283 saturation:1.0 brightness:0.62 alpha:1.0];
		
	// Add the navigation controller's view to the window and show the window
	[self.window addSubview:navigationController.view];
	[self.window makeKeyAndVisible];
}

// The app is about to close down so we have to save its state
- (void)applicationWillTerminate:(UIApplication *)application
{
	// The place where the file should be stored
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString *path = [documentsDirectory stringByAppendingPathComponent:@"LocationPresets.plist"];

	// Find out which game that is playing at which board
	int lottoViewIndex = lottoController.selectedLottoViewIndex;
	NSString *currentBoard = (lottoViewIndex == 0) ? GAME_ONE : GAME_TWO;
	
	// Add the view index to the locations dictionary that will be written to file
	[locationsDictionary setObject:currentBoard forKey:GAME_SEGMENT];
	// Add the current game title to the dictionary
	[locationsDictionary setObject:lottoController.currentTitle forKey:CURRENT_GAME];
	// Add the Lotto Data of the first game to the dictionary
	[locationsDictionary setObject:[[lottoController.lottoOneView data] numbers] forKey:VIEW_ONE_DATA];
	// Add the button matrix status to the dictionary
	[locationsDictionary setObject:[lottoController.lottoOneView cellMatrix] forKey:VIEW_ONE_MATRIX];
	// Repeat for the second game if applicable
	if ([lottoController.lottoTwoView data])
		[locationsDictionary setObject:[[lottoController.lottoTwoView data] numbers] forKey:VIEW_TWO_DATA];
	if ([lottoController.lottoTwoView cellMatrix])
		[locationsDictionary setObject:[lottoController.lottoTwoView cellMatrix] forKey:VIEW_TWO_MATRIX];
	
	// Finally save the dictionary to the file
	int result = [locationsDictionary writeToFile:path atomically:YES];
	if (!result) {
		NSAssert(0, @"Failed to save presets file");
	}
}

- (void)dealloc
{
	[window release];
	[lottoController release];
	[navigationController release];
	[locationsDictionary release];
	[super dealloc];
}

// Creates a writable copy of the bundled default presets in the application Documents directory
- (void)createEditableCopyOfPresetsIfNeeded
{
    BOOL success;
    NSError *error = 0;
    NSFileManager *fileManager = [NSFileManager defaultManager];
	
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *writableGPPath = [documentsDirectory stringByAppendingPathComponent:@"GamePresets.plist"];
	
	// First, test for the existence of a previously created copy
    success = [fileManager fileExistsAtPath:writableGPPath];
    if (success) return;
	
    // The writable presets does not exist, so copy the default to the appropriate location
	NSString *defaultGPPath = [[NSBundle mainBundle] pathForResource:@"GamesPresets" ofType:@"plist"];
    success = [fileManager copyItemAtPath:defaultGPPath toPath:writableGPPath error:&error];
    if (!success) {
        NSAssert(0, @"Failed to create writable presets file");
    }
}

// The app state is stored in a file area plist. Create this here or retrieve an existing file
- (void)createUserLocationPresetsIfNeeded
{
	BOOL success;
    NSFileManager *fileManager = [NSFileManager defaultManager];
	
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *filePath = [documentsDirectory stringByAppendingPathComponent:@"LocationPresets.plist"];
	
	success = [fileManager fileExistsAtPath:filePath];
    if (success) {
		// A file is present in the document directory, create a dictionary from it
		locationsDictionary = [[NSDictionary dictionaryWithContentsOfFile:filePath] retain];
	} else {
		// No location presets found, create default ones here and save as plist
		locationsDictionary = [[[NSMutableDictionary alloc] init] retain];
		// First store the frontmost game
		[locationsDictionary setObject:GAME_ONE forKey:GAME_SEGMENT];
		
		// Then get the name of the first game preset and store that
		NSString *path = [documentsDirectory stringByAppendingPathComponent:@"GamePresets.plist"];
		NSDictionary *gamesDictionary = [NSDictionary dictionaryWithContentsOfFile:path];
		NSString *firstTitle = [[gamesDictionary allKeys] objectAtIndex:0];
		[locationsDictionary setObject:firstTitle forKey:CURRENT_GAME];
		
		// Finally, write the dict as a file
		success = [locationsDictionary writeToFile:filePath atomically:YES];
		if (!success) {
			NSAssert(0, @"Failed to create location presets file");
		}
	}
}

@end
