//
//  LottoViewController.m
//  LottoTouch
//
//  Created by Bertils mini on 2008-03-21.
//  Copyright 2008-2009 Sork Software. All rights reserved.
//

#import "Defines.h"
#import "LottoViewController.h"
#import "GamesViewController.h"
#import "PickerViewController.h"
#import "LottoMatrixView.h"
#import "LottoData.h"


@interface LottoViewController (Private)
- (void)setupGamesFromFile;
- (void)setupGamesFromList;
- (void)setLottoView:(NSInteger)viewIndex;
- (void)setGameLabels;
- (void)updateLottoViewsFromFile;
- (void)updateLottoViewsFromList;
- (void)setupLottoButtons;
- (LottoMatrixView *)lottoView;
- (void)presetsChanged:(NSNotification *)notification;
- (void)matrixChanged:(NSNotification *)notification;
- (void)retrieveGameState;
- (void)retrieveGamesList;
- (void)saveGamesList;
- (void)gameAction:(id)sender;
- (void)lotto:(id)sender;
- (void)clearAction:(id)sender;
@end


@implementation LottoViewController

@synthesize selectedLottoViewIndex;
@synthesize gamesDictionary, locationsDictionary;
@synthesize lottoOneView, lottoTwoView;
@synthesize lotto1Size, sample1Size, lotto2Size, sample2Size;
@synthesize currentTitle;


- (id)init
{
	if (self = [super init]) {
		// Make the title of this page the same as the name of this app
		self.title = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleName"];
		
		[[UIAccelerometer sharedAccelerometer] setUpdateInterval:(1.0 / kUpdateFrequency)];
		[[UIAccelerometer sharedAccelerometer] setDelegate:self];
	}
	return self;
}

- (void)dealloc
{
	[super dealloc];
	[gamesDictionary release];
	[locationsDictionary release];
	[clearButton release];
	[gamesViewController release];
	[gameOneLabel release];
	[gameTwoLabel release];
	[lottoOneButton release];
	[lottoTwoButton release];
	[containerView release];
	[lottoOneView release];
	[lottoTwoView release];
	[notificationCenter removeObserver:self];
}

// Create and set up the content of the main Lotto view, top to bottom
- (void)loadView
{
	// Create the base for a custom view hierarchy – the topmost parent view
	UIView *contentView = [[UIView alloc] initWithFrame:[UIScreen mainScreen].applicationFrame];
	
	// Gray background of the game label
	contentView.backgroundColor = [UIColor colorWithHue:0.0 saturation:0.0 brightness:0.69 alpha:1.0];
	
	self.view = contentView;
	[contentView release];

	// Make a custom "Clear" bar button for the navigation bar's left view
	clearButton = [[UIBarButtonItem alloc] 
					initWithTitle:NSLocalizedString(@"ClearButtonTitle", @"Clear lotto matrix of 'clusions")
					style:UIBarButtonItemStylePlain
					target:self
					action:@selector(clearAction:)];
	// Button will be added when needed, clear view for now
	[self.navigationItem setLeftBarButtonItem:nil animated:NO];
		
	// Add a "Games" bar button to the right view of the navigation bar
	UIBarButtonItem *gameButton = [[UIBarButtonItem alloc]
									initWithTitle:NSLocalizedString(@"GamesButtonTitle", @"Lotto page right Games bar button")
									style:UIBarButtonItemStyleBordered
									target:self
									action:@selector(game:)];
	self.navigationItem.rightBarButtonItem = gameButton;
	[gameButton release];
	
	// Add a label for the primary game (second label added lazily when needed)
	CGRect labelFrame = CGRectMake(kLeftMargin,
								   kTitleBarHeight - kTextFieldHeight,
								   self.view.bounds.size.width - (kRightMargin * 2.0),
								   kTextFieldHeight);
	gameOneLabel = [[UILabel alloc] initWithFrame:labelFrame];
    gameOneLabel.font = [UIFont boldSystemFontOfSize:18];
	gameOneLabel.textAlignment = UITextAlignmentCenter;
	gameOneLabel.textColor = [UIColor whiteColor];
	gameOneLabel.shadowColor = [UIColor grayColor];
	gameOneLabel.shadowOffset = CGSizeMake(0, -1);
	gameOneLabel.backgroundColor = [UIColor clearColor];
	[gameOneLabel setAlpha:1];
	[self.view addSubview:gameOneLabel];

	// Create a container view for a flip animation
	CGRect frame = CGRectMake(0.0, kTitleBarHeight, klottoWidth, klottoHeight);
	containerView = [[UIView alloc] initWithFrame:frame];
	[self.view addSubview:containerView];
	
	// First create the secondary lotto view
	frame = CGRectMake(0.0, 0.0, klottoWidth, klottoHeight);
	lottoTwoView = [[LottoMatrixView alloc] initWithFrame:frame];
	[lottoTwoView setBackgroundColor:[UIColor colorWithRed:0.97 green:0.80 blue:0.29 alpha:1.0]];
	[containerView addSubview:lottoTwoView];
	
	// Then create the primary lotto view
	frame = CGRectMake(0.0, 0.0, klottoWidth, klottoHeight);
	lottoOneView = [[LottoMatrixView alloc] initWithFrame:frame];
	[lottoOneView setBackgroundColor:[UIColor colorWithHue:0.125 saturation:0.71 brightness:0.98 alpha:1.0]];
	[containerView addSubview:lottoOneView];

	// Place a green background bar for the lotto buttons at the bottom of the screen
	UIImage* buttonBarImage = [UIImage imageNamed:@"buttonBarGreen.png"];
	CGRect buttonBarFrame = CGRectMake(0,
									   kScreenHeight - kStatusBarHeight - kToolBarHeight - kButtonBarHeight,
									   kScreenWidth,
									   kButtonBarHeight);
	buttonBarView = [[UIImageView alloc] initWithFrame:buttonBarFrame];
	[buttonBarView setImage:buttonBarImage];
	[self.view addSubview:buttonBarView];
	[buttonBarView release];

	// Add a full width Lotto button for the most common case of a single game
	CGRect lottoOneFrame = CGRectMake(13, 
									  kScreenHeight - kStatusBarHeight - kToolBarHeight - kButtonBarHeight + kTopMargin,
									  kScreenWidth - 26,
									  kStdButtonHeight);	
	lottoOneButton = [[UIButton buttonWithType:UIButtonTypeRoundedRect] retain];
	lottoOneButton.frame = lottoOneFrame;
	lottoOneButton.contentVerticalAlignment = UIControlContentVerticalAlignmentCenter;
	lottoOneButton.contentHorizontalAlignment = UIControlContentHorizontalAlignmentCenter;
	[lottoOneButton setTitle:NSLocalizedString(@"LottoButtonTitle", @"LOTTO button") forState:UIControlStateNormal];
	[lottoOneButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
	[lottoOneButton setTitleShadowColor:[UIColor grayColor] forState:UIControlStateNormal];
	//lottoButton.reversesTitleShadowWhenHighlighted = YES;
	lottoOneButton.titleLabel.shadowOffset = CGSizeMake(0.0, -1.0);
	lottoOneButton.titleLabel.font = [UIFont boldSystemFontOfSize:20];
	
	UIImage* lottoOneButtonImage = [UIImage imageNamed:@"greenUntouchedProto.png"];
	UIImage *wideLottoOneButtonImage = [lottoOneButtonImage stretchableImageWithLeftCapWidth:20.0 topCapHeight:0.0];
	[lottoOneButton setBackgroundImage:wideLottoOneButtonImage forState:UIControlStateNormal];
	
	UIImage* pressedButtonOneImage = [UIImage imageNamed:@"greenTouchedProto.png"];
	UIImage *widePressedButtonOneImage = [pressedButtonOneImage stretchableImageWithLeftCapWidth:20.0 topCapHeight:0.0];
	[lottoOneButton setBackgroundImage:widePressedButtonOneImage forState:UIControlStateHighlighted];	
	[lottoOneButton addTarget:self action:@selector(lotto:) forControlEvents:UIControlEventTouchUpInside];
	[self.view addSubview:lottoOneButton];
		
	// Get the saved game state with the selected index and current title
	[self retrieveGameState];
	
	// Which gamelet should be the frontmost? 
	NSString *segment = [locationsDictionary objectForKey:GAME_SEGMENT];
	selectedLottoViewIndex = ([segment isEqualToString:GAME_ONE]) ? 0 : 1;
	
	// Which game did the user select?
	self.currentTitle = [locationsDictionary objectForKey:CURRENT_GAME];	
	
	// Listen for game view touchesEnded that should enable the Clear button
	[notificationCenter addObserver:self selector:@selector(matrixChanged:)
							   name:SSLottoTouchMatrixChangedNotification object:nil];			
	
	// Setup flip view, update titles, populate the lotto views, and setup the main buttons using saved values if possible
	[self setupGamesFromFile];
}

// Called by loadView, will look for saved game data
- (void)setupGamesFromFile
{
	// Set up the flip view with the correct view order
	[self setLottoView:selectedLottoViewIndex];
	
	// Set up the view(s) with the relevant game data
	[self updateLottoViewsFromFile];

	// Update the game title
	[self setGameLabels];
		
	// Setup the button bar with one or two buttons
	[self setupLottoButtons];
	
	// This is the front lotto view
	LottoMatrixView *frontView = [self lottoView];

	// Enable the Clear button if the front view has been touched
	// Also refresh the 'clusions counts from the button matrix
	if ([frontView hasBeenTouched]) {
		[self.navigationItem setLeftBarButtonItem:clearButton animated:YES];
	} else {
		[self.navigationItem setLeftBarButtonItem:nil animated:YES];
	}
	// Finally, update the view
	[frontView setNeedsDisplay];
}

// Called by presetsChanged: using data from game settings
- (void)setupGamesFromList
{
	// Set up the flip view with the correct view order
	[self setLottoView:selectedLottoViewIndex];
	
	// Set up the view(s) with the game data from list
	[self updateLottoViewsFromList];

	// Update the game title
	[self setGameLabels];
		
	// Setup the button bar with one or two buttons
	[self setupLottoButtons];
	
	// Update the front lotto view
	LottoMatrixView *frontView = [self lottoView];
	[frontView setNeedsDisplay];
}

// Swap lotto views in and out of flipview. Called by setupGamesFromFile/List, gameAction:
- (void)setLottoView:(NSInteger)viewIndex
{
	if ((viewIndex == 0) && [lottoTwoView superview])
	{
		[lottoTwoView removeFromSuperview];
		[containerView addSubview:lottoOneView];
	}
	if ((viewIndex == 1) && [lottoOneView superview])
	{
		[lottoOneView removeFromSuperview];
		[containerView addSubview:lottoTwoView];
	}	
}

// Only one label is visible at a time. Called by setupGamesFromFile/List
- (void)setGameLabels
{
	// Need a second label?
	if (!game2Enabled) {
		// No, just use the game title
		gameOneLabel.text = self.currentTitle;
	} else {
		// Yes, do we have the second label?
		if (!gameTwoLabel) {
			// No, create it lazily here
			CGRect labelFrame = CGRectMake(kLeftMargin,
										   kTitleBarHeight - kTextFieldHeight,
										   self.view.bounds.size.width - (kRightMargin * 2.0),
										   kTextFieldHeight);			
			gameTwoLabel = [[UILabel alloc] initWithFrame:labelFrame];
			gameTwoLabel.font = [UIFont boldSystemFontOfSize:18];
			gameTwoLabel.textAlignment = UITextAlignmentCenter;
			gameTwoLabel.textColor = [UIColor whiteColor];
			gameTwoLabel.shadowColor = [UIColor grayColor];
			gameTwoLabel.shadowOffset = CGSizeMake(0, -1);
			gameTwoLabel.backgroundColor = [UIColor clearColor];
			[gameTwoLabel setAlpha:0];
			[self.view addSubview:gameTwoLabel];			
		}
		// Set both labels with game number suffix
		gameOneLabel.text = [self.currentTitle stringByAppendingString:NSLocalizedString(@"game1label", @"Game 1 title suffix")];
		gameTwoLabel.text = [self.currentTitle stringByAppendingString:NSLocalizedString(@"game2label", @"Game 2 title suffix")];
	}
	// Show the proper label
	[gameOneLabel setAlpha:selectedLottoViewIndex ? 0.0 : 1.0];
	[gameTwoLabel setAlpha:selectedLottoViewIndex ? 1.0 : 0.0];
}

// Called by setupGamesFromFile
- (void)updateLottoViewsFromFile
{
	// Get the saved games dictionary data from file
	[self retrieveGamesList];

	// Get the dict for the current title
	NSDictionary *game = [gamesDictionary objectForKey:self.currentTitle];

	// Get the parameters from the dict for the current game title
	self.lotto1Size = [[game objectForKey:LOTTO1_SIZE] intValue];
	self.sample1Size = [[game objectForKey:SAMPLE1_SIZE] intValue];
	self.lotto2Size = [[game objectForKey:LOTTO2_SIZE] intValue];
	self.sample2Size = [[game objectForKey:SAMPLE2_SIZE] intValue];

	// First create a new lotto data object
	LottoData *data = [LottoData lottoDataWithCount:self.lotto1Size];

	// Then try to get a stored lotto data array from the locations file
	NSMutableArray *dataArray = [locationsDictionary objectForKey:VIEW_ONE_DATA];

	if (dataArray) {
		// Finally, replace the lotto object's data array if applicable
		[data setNumbers:dataArray];
		[dataArray release];
	}
    // Turn the data over to the view object which will retain it
    [lottoOneView setData:data];
	
	// Set the parameters for the game
	lottoOneView.lottoSize = self.lotto1Size;
	lottoOneView.sampleSize = self.sample1Size;
	
	// Prepare the button matrix default content
	[lottoOneView updateBoard];
	
	// Try to get a stored cellmatrix from the locations file
	NSMutableArray *cellArray = [locationsDictionary objectForKey:VIEW_ONE_MATRIX];

	if (cellArray) {
		// Replace the default matrix with the stored one
		[lottoOneView setCellMatrix:cellArray];
		[cellArray release];
	}
	
	// Do we need a second game?
	game2Enabled = (self.lotto2Size > 0) ? YES : NO;
	
	// Populate the second view, if required
	if (game2Enabled) {
		// Get stored data for second view, if available
		data = [LottoData lottoDataWithCount:self.lotto2Size];
		dataArray = [locationsDictionary objectForKey:VIEW_TWO_DATA];
		if (dataArray) {
			[data setNumbers:dataArray];
			[dataArray release];
		}		
		[lottoTwoView setData:data];
		lottoTwoView.lottoSize = self.lotto2Size;
		lottoTwoView.sampleSize = self.sample2Size;
		[lottoTwoView updateBoard];
		cellArray = [locationsDictionary objectForKey:VIEW_TWO_MATRIX];
		if (cellArray) {
			[lottoTwoView setCellMatrix:cellArray];
			[cellArray release];
		}
	} else {
		// No second view
		[lottoTwoView setData:nil];
		lottoTwoView.lottoSize = self.lotto2Size;
		lottoTwoView.sampleSize = self.sample2Size;		
	}
}

// Called by setupGamesFromList
- (void)updateLottoViewsFromList
{
	// Get the saved games dictionary data from file
	[self retrieveGamesList];
	
	// Get the dict for the current title
	NSDictionary *game = [gamesDictionary objectForKey:self.currentTitle];
	
	// Get the parameters from the dict for the current game title
	self.lotto1Size = [[game objectForKey:LOTTO1_SIZE] intValue];
	self.sample1Size = [[game objectForKey:SAMPLE1_SIZE] intValue];
	self.lotto2Size = [[game objectForKey:LOTTO2_SIZE] intValue];
	self.sample2Size = [[game objectForKey:SAMPLE2_SIZE] intValue];
	
	// First create a new lotto data object
	LottoData *data = [LottoData lottoDataWithCount:self.lotto1Size];
	// Turn the data over to the view class which will retain it
    [lottoOneView setData:data];
	
	// Set the parameters for the game
	lottoOneView.lottoSize = self.lotto1Size;
	lottoOneView.sampleSize = self.sample1Size;
	
	// Prepare the button matrix default content
	[lottoOneView updateBoard];
	
	// Scramble the data
	[[lottoOneView data] randomize];
	
	// Disable the Clear button
	[self.navigationItem setLeftBarButtonItem:nil animated:NO];
	
	// Need a second game?
	game2Enabled = (self.lotto2Size > 0) ? YES : NO;
	
	// Populate the second view, if required
	if (game2Enabled) {
		data = [LottoData lottoDataWithCount:self.lotto2Size];
		[lottoTwoView setData:data];
		lottoTwoView.lottoSize = self.lotto2Size;
		lottoTwoView.sampleSize = self.sample2Size;
		[lottoTwoView updateBoard];
		[[lottoTwoView data] randomize];
	} else {
		// No second view
		[lottoTwoView setData:nil];
		lottoTwoView.lottoSize = self.lotto2Size;
		lottoTwoView.sampleSize = self.sample2Size;		
	}
}

// Called by setupGamesFromFile/List
- (void)setupLottoButtons
{
	// Is second game enabled?
	if (game2Enabled) {
		// Yes, return if bar already have two visible buttons
		if (lottoTwoButton && ![lottoTwoButton isHidden]) return;
		
		// No, show two buttons, creating the second if required
		else {
			// Do we have a second button?
			if (!lottoTwoButton) {
				// No, create the second button here
				CGRect lottoTwoFrame = CGRectMake(kLeftMargin - 1.0, 
												  kScreenHeight - kStatusBarHeight - kToolBarHeight - kButtonBarHeight + kTopMargin,
												  (kScreenWidth - 26) / 2,
												  kStdButtonHeight);
				lottoTwoButton = [[UIButton buttonWithType:UIButtonTypeRoundedRect] retain];
				lottoTwoButton.frame = lottoTwoFrame;
				lottoTwoButton.contentVerticalAlignment = UIControlContentVerticalAlignmentCenter;
				lottoTwoButton.contentHorizontalAlignment = UIControlContentHorizontalAlignmentCenter;
				[lottoTwoButton setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
				[lottoTwoButton setTitleShadowColor:[UIColor grayColor] forState:UIControlStateNormal];
				lottoTwoButton.titleLabel.shadowOffset = CGSizeMake(0.0, -1.0);
				lottoTwoButton.titleLabel.font = [UIFont boldSystemFontOfSize:20];
				
				UIImage* lottoOneButtonTwoImage = [UIImage imageNamed:@"leftGrayArrowUntouched.png"];
				UIImage *wideLottoTwoButtonImage = [lottoOneButtonTwoImage stretchableImageWithLeftCapWidth:20.0 topCapHeight:0.0];
				[lottoTwoButton setBackgroundImage:wideLottoTwoButtonImage forState:UIControlStateNormal];
				
				UIImage* pressedButtonTwoImage = [UIImage imageNamed:@"leftGrayArrowTouched.png"];
				UIImage *widePressedButtonTwoImage = [pressedButtonTwoImage stretchableImageWithLeftCapWidth:20.0 topCapHeight:0.0];
				[lottoTwoButton setBackgroundImage:widePressedButtonTwoImage forState:UIControlStateHighlighted];	
				[lottoTwoButton addTarget:self action:@selector(gameAction:) forControlEvents:UIControlEventTouchUpInside];
				[self.view addSubview:lottoTwoButton];
			}
			// Show second button if hidden
			lottoTwoButton.hidden = NO;
			
			// Make the first (right) button short
			CGRect lottoOneFrame = CGRectMake((self.view.bounds.size.width - 26) / 2 + 17, 
											  kScreenHeight - kStatusBarHeight - kToolBarHeight - kButtonBarHeight + kTopMargin,
											  (kScreenWidth - 26) / 2,
											  kStdButtonHeight);	
			lottoOneButton.frame = lottoOneFrame;
		}
		// Update the text on the swap button to point to the other game
		NSString *lottoTwoButtonText = [NSLocalizedString(@"lotto2button", @"Lotto 2 button Game title")
										stringByAppendingFormat:@"%u", selectedLottoViewIndex ? 1 : 2];
		[lottoTwoButton setTitle:lottoTwoButtonText forState:UIControlStateNormal];
	}
	// Second game was not enabled
	else {
		// Hide the second button
		lottoTwoButton.hidden = YES;
		
		// And make the first (right) button long
		CGRect lottoOneFrame = CGRectMake(13, 
										  kScreenHeight - kStatusBarHeight - kToolBarHeight - kButtonBarHeight + kTopMargin,
										  kScreenWidth - 26,
										  kStdButtonHeight);	
		lottoOneButton.frame = lottoOneFrame;
	}
}

#pragma mark Utility methods

// The current Lotto View
- (LottoMatrixView *)lottoView {
	return selectedLottoViewIndex ? lottoTwoView : lottoOneView;
}

#pragma mark -
#pragma mark Accelerometer support - from Apple GLPaint example

// Called when the accelerometer detects motion; play Lotto if the motion is over a threshold
- (void)accelerometer:(UIAccelerometer*)accelerometer didAccelerate:(UIAcceleration*)acceleration
{
	UIAccelerationValue	length, x, y, z;
	
	// Use a basic high-pass filter to remove the influence of the gravity
	myAccelerometer[0] = acceleration.x * kFilteringFactor + myAccelerometer[0] * (1.0 - kFilteringFactor);
	myAccelerometer[1] = acceleration.y * kFilteringFactor + myAccelerometer[1] * (1.0 - kFilteringFactor);
	myAccelerometer[2] = acceleration.z * kFilteringFactor + myAccelerometer[2] * (1.0 - kFilteringFactor);
	
	// Compute values for the three axes of the acceleromater
	x = acceleration.x - myAccelerometer[0];
	y = acceleration.y - myAccelerometer[0];
	z = acceleration.z - myAccelerometer[0];
	
	// Compute the intensity of the current acceleration 
	length = sqrt(x * x + y * y + z * z);
	
	// If above a given threshold, play the game
	if (length >= kAccelerationThreshold) {
		[self lotto:self];
	}
}

#pragma mark -
#pragma mark Notifications

// Handle selections in the Games view here
- (void)presetsChanged:(NSNotification *)notification
{
	// Get the change type from the notification userInfo
	NSString *prefsType = [[notification userInfo] objectForKey:@"Change"];
	
	if ([prefsType isEqualToString:@"Game"]) {
		// User have chosen a new game title
		self.currentTitle = gamesViewController.selectedTitle;
		
		// The games list may have been changed
		self.gamesDictionary = gamesViewController.gamesDictionary;
		
		// Save the changed games list
		[self saveGamesList];
		
		// Reset the flip views to the first
		selectedLottoViewIndex = 0;
		
		// Setup flip view, update title and button, and populate the views for the choosen game
		[self setupGamesFromList];
	}
}

// Control the 'clusions Clear button here
- (void)matrixChanged:(NSNotification *)notification
{
	// Get the change type from the notification userInfo
	NSString *prefsType = [[notification userInfo] objectForKey:@"Touch"];
	
	if ([prefsType isEqualToString:@"Enable"]) {
		[self.navigationItem setLeftBarButtonItem:clearButton animated:YES];
	} else {
		[self.navigationItem setLeftBarButtonItem:nil animated:YES];
	}
}

#pragma mark -
#pragma mark Data retrieval & storage

// Get the saved game state params. Called by loadView
- (void)retrieveGameState
{
	// Get the path to the parameter plist in the user's sandbox
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *path = [documentsDirectory stringByAppendingPathComponent:@"LocationPresets.plist"];
	
	// Did a path to a valid file exist?
	if (!path) {
		NSAssert(0, @"Failed to find location presets file");
	}
	// Yes, get a dictionary from the plist
	locationsDictionary = [[NSDictionary dictionaryWithContentsOfFile:path] retain];
}

// Get the items in the games list. Called by updateLottoViews (called by setupGames)
- (void)retrieveGamesList
{
	// Use a write enabled plist copy in the app's document directory to get the games list
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths objectAtIndex:0];
    NSString *path = [documentsDirectory stringByAppendingPathComponent:@"GamePresets.plist"];
	
	// Did a path to a valid copy exist?
	if (!path) {
		NSAssert(0, @"Failed to find games presets file");
	}
	// Yes, create a dictionary from the plist
	gamesDictionary = [[NSMutableDictionary dictionaryWithContentsOfFile:path] retain];
}

// Save an updated games list. Called by presetsChanged:
- (void)saveGamesList
 {
	// Save the games dictionary, first get the path
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString *path = [documentsDirectory stringByAppendingPathComponent:@"GamePresets.plist"];
	 
	// Then actually write the plist to memory
	BOOL result = [gamesDictionary writeToFile:path atomically:YES];
	if (!result) {
		NSAssert(0, @"Failed to save games presets file");
	}
 }

#pragma mark -
#pragma mark Action methods

// Animate between the two games if a second one is enabled
- (void)gameAction:(id)sender
{
	// Change index to other view
	selectedLottoViewIndex = !selectedLottoViewIndex;

	// Arrow button should address the other view
	NSString *lottoTwoButtonText = [NSLocalizedString(@"lotto2button", @"Lotto 2 button Game title")
									stringByAppendingFormat:@"%u", selectedLottoViewIndex ? 1 : 2];
	[lottoTwoButton setTitle:lottoTwoButtonText forState:UIControlStateNormal];
	
	// Default animation curve is UIViewAnimationCurveEaseInOut
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:kTransitionDuration];
	[UIView setAnimationTransition:UIViewAnimationTransitionFlipFromRight
						   forView:containerView cache:YES];
	
	// Actually swap the views
	[self setLottoView:selectedLottoViewIndex];
		
	// Fade view label text out and in 
	[gameOneLabel setAlpha:selectedLottoViewIndex ? 0.0 : 1.0];
	[gameTwoLabel setAlpha:selectedLottoViewIndex ? 1.0 : 0.0];
	
	[UIView commitAnimations];
	
	// Get the front lotto view
	LottoMatrixView *frontView = [self lottoView];
	
	// Enable Clear button if view has been touched
	if ([frontView hasBeenTouched]) {
		[self.navigationItem setLeftBarButtonItem:clearButton animated:YES];
	} else {
		[self.navigationItem setLeftBarButtonItem:nil animated:YES];
	}
	[frontView setNeedsDisplay];
}

// Actually start a lotto game here
- (void)lotto:(id)sender
{
	LottoMatrixView *frontView = [self lottoView];
	// Scramble the data array
	[[frontView data] randomize];
    [frontView setNeedsDisplay];
}

// Transfer to the games table view, creating this as needed
- (void)game:(id)sender
{
	if (!gamesViewController) {
		gamesViewController = [[GamesViewController alloc] init];
		
		// Listen to updates from the games view
		[notificationCenter addObserver:self selector:@selector(presetsChanged:)
								   name:SSLottoTouchPresetsChangedNotification object:nil];		
	}
	// Let the controller know about the available games
	gamesViewController.gamesDictionary = self.gamesDictionary;
	// and the current game
	gamesViewController.selectedTitle = self.currentTitle;
	
	// Reset any includes or excludes
	[self clearAction:self];
	
	// Then go to the games list
	[self.navigationController pushViewController:gamesViewController animated:YES];
}

// Clear the playing field of inclusions or exclusions
- (void)clearAction:(id)sender
{	
	LottoMatrixView *frontView = [self lottoView];
	// Hide Clear button
	[self.navigationItem setLeftBarButtonItem:nil animated:YES];
	// Clear the button matrix
	[frontView resetBoard];
    [frontView setNeedsDisplay];
}

@end
