//
//  PickerViewController.m
//  LottoTouch
//
//  Created by Bertils mini on 2008-03-29.
//  Copyright Sork Software 2008-2009. All rights reserved.
//

#import "PickerViewController.h"
#import "TitleViewController.h"
#import "Defines.h"

NSString *SSLottoTouchPresetsChangedNotification = @"Picker presets changed";


@interface PickerViewController (Private)
- (void)cancelAction:(id)sender;
- (void)saveAction:(id)sender;
- (void)deleteAction:(id)sender;
- (void)presetsChanged:(NSNotification *)notification;
@end


@implementation PickerViewController

@synthesize gameTitle, newGame, lotto1Size, sample1Size, lotto2Size, sample2Size;


- (id)init
{
	if (self = [super init])
	{
		// This will appear as the title in the navigation bar
		self.title = NSLocalizedString(@"PickerViewTitle", @"");
	}
	return self;
}

- (void)dealloc
{
	[titleTableView release];
	[myPickerView release];
	[titleViewController release];
	[notificationCenter removeObserver:self];
	[super dealloc];
}

- (void)loadView
{
	// Setup the content view
	UIView *contentView = [[UIView alloc] initWithFrame:[[UIScreen mainScreen] applicationFrame]];

	// Use the table view background color
	contentView.backgroundColor = [UIColor groupTableViewBackgroundColor];
	
	// Add it as the parent/content view to this UIViewController
	self.view = contentView;
	[contentView release];

	// Add a Cancel bar button in the nav bar's left view for a return to the game
	UIBarButtonItem *cancelButton = [[UIBarButtonItem alloc]
									initWithTitle:NSLocalizedString(@"CancelButtonTitle", @"")
									style:UIBarButtonItemStyleDone
									target:self
									action:@selector(cancelAction:)];
	self.navigationItem.leftBarButtonItem = cancelButton;
	[cancelButton release];
	
	// Add a Save bar button in the nav bar's right view to save new picks
	UIBarButtonItem *saveButton = [[UIBarButtonItem alloc]
									  initWithTitle:NSLocalizedString(@"SaveButtonTitle", @"")
									  style:UIBarButtonItemStyleDone
									  target:self
									  action:@selector(saveAction:)];
	self.navigationItem.rightBarButtonItem = saveButton;
	[saveButton release];
	
	// Create a short table view for the game title
	CGRect tableFrame = CGRectMake(kLeftMargin,
								   kTopMargin,
								   self.view.bounds.size.width - (kRightMargin * 2.0),
								   kToolBarHeight * 2.0);	
    UITableView *aTableView = [[[UITableView alloc] initWithFrame:tableFrame style:UITableViewStyleGrouped] autorelease];
    titleTableView = aTableView;
    titleTableView.delegate = self;
    titleTableView.dataSource = self;
    [self.view addSubview:titleTableView];
	
	// Add the picker at the screen bottom
	myPickerView = [[UIPickerView alloc] initWithFrame:CGRectZero];
	CGSize pickerSize = [myPickerView sizeThatFits:CGSizeZero];
	CGRect pickerRect = CGRectMake(	0.0, 
								   self.view.bounds.size.height - kToolBarHeight - pickerSize.height,
								   pickerSize.width,
								   pickerSize.height);
	myPickerView.frame = pickerRect;
	myPickerView.delegate = self;
	myPickerView.showsSelectionIndicator = YES;	// defaults to NO – revealed in beta 4
	[self.view addSubview:myPickerView];

	// Labels for picker settings
	CGRect labelFrame = CGRectMake(kLeftMargin,
								   myPickerView.frame.origin.y - kTextFieldHeight,
								   self.view.bounds.size.width - (kRightMargin * 2.0),
								   kTextFieldHeight);	
	UILabel *gameLabel = [[UILabel alloc] initWithFrame:labelFrame];
	gameLabel.font = [UIFont systemFontOfSize: 18];
	gameLabel.textAlignment = UITextAlignmentCenter;
	gameLabel.textColor = [UIColor blackColor];
	gameLabel.shadowColor = [UIColor whiteColor];
	gameLabel.shadowOffset = CGSizeMake(0, 1);
	gameLabel.backgroundColor = [UIColor clearColor];
	gameLabel.text = NSLocalizedString(@"gameLabel", @"Picker view game settings label");
	[self.view addSubview:gameLabel];
	[gameLabel release];
}

#pragma mark -
#pragma mark Notifications

// Handle changes in the Title view here
- (void)presetsChanged:(NSNotification *)notification
{
	// Get the change type from the notification userInfo
	NSString *prefsType = [[notification userInfo] objectForKey:@"Change"];
	
	// Catch a new title here, it is subsequently saved by the GamesViewController
	if ([prefsType isEqualToString:@"Title"]) {
		self.gameTitle = titleViewController.gameTitle;
		// We have a title, no longer a new game...
		self.newGame = NO;
	}
}

#pragma mark -
#pragma mark UITableView delegate methods

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	// Create the Title view lazily
	if (titleViewController == nil) {
		titleViewController = [[TitleViewController alloc] init];
		
		// Listen to updates from the title view
		[notificationCenter addObserver:self selector:@selector(presetsChanged:)
								   name:SSLottoTouchPresetsChangedNotification object:nil];
	}
	
	// Title view should have the current game title
	titleViewController.gameTitle = self.gameTitle;
	// A special case
	titleViewController.newGame = self.newGame;
	
	// Push the title view on to the navigation controller's stack
	[self.navigationController pushViewController:titleViewController animated:YES];
}

#pragma mark UITableView data source methods

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	return 1;
}

// Only called on the first visit to this view!
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	UITableViewCell *cell = (UITableViewCell*)[tableView dequeueReusableCellWithIdentifier:CELL_IDENTIFIER];
	
	if (cell == nil)
	{
		cell = [[[UITableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:CELL_IDENTIFIER] autorelease];
		
		// Set fixed cell attributes – new in 3.0
		cell.textLabel.textColor = [UIColor darkTextColor];
		cell.accessoryType =  UITableViewCellAccessoryDisclosureIndicator;
	}
	
	// Set variable cell attributes – eg the text label text – new in 3.0
	if (self.gameTitle) {
		cell.textLabel.text = self.gameTitle;
	} else {
		cell.textLabel.text = NSLocalizedString(@"NewTitle", @"Picker View Table View new title placeholder text");
	}

	return cell;
}

#pragma mark -
#pragma mark UIPickerView delegate methods

// Make sure the picker settings are relevant and notify the Games controller about changes
- (void)pickerView:(UIPickerView *)pickerView didSelectRow:(NSInteger)row inComponent:(NSInteger)component
{
	switch (component)
	{
		case 0:
		{
			// Ensure a Game 1 minimum sample size
			if (row >= SAMPLE_SIZE_MIN) {
				self.lotto1Size = row;
			} else {
				self.lotto1Size = SAMPLE_SIZE_MIN;
				[myPickerView selectRow:SAMPLE_SIZE_MIN inComponent:0 animated:YES];
			}
			break;
		}
		case 1:
		{
			// The Game 1 sample must be at least 1
			if (row == 0) {
				self.sample1Size = 1;
				[myPickerView selectRow:self.sample1Size inComponent:1 animated:YES];
			} else {
				self.sample1Size = row;
			}
			break;
		}
		case 2:
		{
			// The Game 2 size is either 0 for no game or at least the minimum size
			if (row >= SAMPLE_SIZE_MIN) {
				self.lotto2Size = row;
			} else {
				if (row < self.lotto2Size) {
					self.lotto2Size = 0;
					self.sample2Size = 0;
					[myPickerView selectRow:self.lotto2Size inComponent:2 animated:YES];
					[myPickerView selectRow:self.sample2Size inComponent:3 animated:YES];
				} else {
					self.lotto2Size = SAMPLE_SIZE_MIN;
					self.sample2Size = 1;
					[myPickerView selectRow:self.lotto2Size inComponent:2 animated:YES];
					[myPickerView selectRow:self.sample2Size inComponent:3 animated:YES];
				}
			}
			break;
		}
		case 3:
		{
			// The Game 2 sample is 0 for no game or at least 1
			if ( self.lotto2Size != 0) {
				if (row == 0) {
					self.sample2Size = 1;
					[myPickerView selectRow:self.sample2Size inComponent:3 animated:YES];
				} else {
					self.sample2Size = row;
				}
			} else {
				self.sample2Size = 0;
				[myPickerView selectRow:self.sample2Size inComponent:3 animated:YES];
			}
			break;
		}
	}
	
	// Let the games controller know about the change
	NSDictionary *presetsChanged = [NSDictionary dictionaryWithObject:@"Picker" forKey:@"Change"];

	[notificationCenter postNotificationName:SSLottoTouchPresetsChangedNotification
					object:self userInfo:presetsChanged];
}

// The picker view text follows the row index directly
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{	
	return [[NSNumber numberWithInt:row] stringValue];
}

- (CGFloat)pickerView:(UIPickerView *)pickerView widthForComponent:(NSInteger)component
{
	CGFloat componentWidth;
	switch (component)
	{
		case 0:
		{
			componentWidth = 80.0;
			break;
		}
		case 1:
		{
			componentWidth = 60.0;
			break;
		}
		case 2:
		{
			componentWidth = 80.0;
			break;
		}
		case 3:
		{
			componentWidth = 60.0;
			break;
		}
	}
	return componentWidth;	
}

#pragma mark UIPickerView data source methods

// This method determines the maximum picker wheel settings
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
	NSInteger numberOfRows;
	switch (component)
	{
		case 0:
		{
			numberOfRows = 100;
			break;
		}
		case 1:
		{
			numberOfRows = SAMPLE_SIZE_MIN;
			break;
		}
		case 2:
		{
			numberOfRows = 100;
			break;
		}
		case 3:
		{
			numberOfRows = SAMPLE_SIZE_MIN;
			break;
		}
	}
	return numberOfRows;
}

- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
	return 4;
}

#pragma mark -
#pragma mark Action methods

// Just return without saving
- (void)cancelAction:(id)sender
{
	[self.navigationController popViewControllerAnimated:YES];
}

// Pop back to the Games view that will save the changes
- (void)saveAction:(id)sender
{	
	NSDictionary *presetsChanged = [NSDictionary dictionaryWithObject:@"Save" forKey:@"Change"];

	[notificationCenter postNotificationName:SSLottoTouchPresetsChangedNotification
									  object:self userInfo:presetsChanged];

	[self.navigationController popViewControllerAnimated:YES];
}

#pragma mark -
#pragma mark UIViewController methods

- (void)viewWillAppear:(BOOL)animated
{
	// Table has only one row
	NSIndexPath *tableRow = [NSIndexPath indexPathForRow:0 inSection:0];
	// Deselect after a visit to the Title view
	[titleTableView deselectRowAtIndexPath:tableRow animated:YES];

	// Title set here on second and subsequent visits to the view – if required
	if (self.gameTitle) {
		UITableViewCell *tableCell = [titleTableView cellForRowAtIndexPath:tableRow];
		// Cell is null on first appearance
		if (tableCell) {
			tableCell.textLabel.text = self.gameTitle;
		}
	}
}

// Update the picker wheels to the current game settings
- (void)viewDidAppear:(BOOL)animated
{	
	[myPickerView selectRow:self.lotto1Size inComponent:0 animated:YES];
	[myPickerView selectRow:self.sample1Size inComponent:1 animated:YES];
	[myPickerView selectRow:self.lotto2Size inComponent:2 animated:YES];
	[myPickerView selectRow:self.sample2Size inComponent:3 animated:YES];
}

@end
